package com.minres.rdl.generator;

import com.google.common.base.Objects;
import com.minres.rdl.RdlUtil;
import com.minres.rdl.rdl.ComponentDefinition;
import com.minres.rdl.rdl.ComponentDefinitionType;
import com.minres.rdl.rdl.ComponentInstance;
import com.minres.rdl.rdl.ExplicitPropertyAssignment;
import com.minres.rdl.rdl.Instantiation;
import com.minres.rdl.rdl.PropertyAssignment;
import com.minres.rdl.rdl.PropertyEnum;
import com.minres.rdl.rdl.RValueConstant;
import java.util.List;
import org.eclipse.emf.common.util.EList;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;

@SuppressWarnings("all")
public class ModuleGenerator extends RdlBaseGenerator {
  private final ComponentDefinition componentDefinition;
  
  public ModuleGenerator(final ComponentDefinition definition) {
    this.componentDefinition = definition;
  }
  
  @Override
  public boolean getOverwrite() {
    return false;
  }
  
  @Override
  public String generateHeader(final String namespace) {
    String _xblockexpression = null;
    {
      ComponentDefinitionType _type = this.componentDefinition.getType();
      boolean _notEquals = (!Objects.equal(_type, ComponentDefinitionType.REGFILE));
      if (_notEquals) {
        return "";
      }
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("/*");
      _builder.newLine();
      _builder.append(" ");
      _builder.append("* Copyright (c) 2019 -2022 MINRES Technologies GmbH");
      _builder.newLine();
      _builder.append(" ");
      _builder.append("*");
      _builder.newLine();
      _builder.append(" ");
      _builder.append("* SPDX-License-Identifier: Apache-2.0");
      _builder.newLine();
      _builder.append(" ");
      _builder.append("*/");
      _builder.newLine();
      _builder.append(" ");
      _builder.newLine();
      _builder.append("#pragma once");
      _builder.newLine();
      _builder.newLine();
      _builder.append("#include <scc/tlm_target.h>");
      _builder.newLine();
      _builder.newLine();
      {
        String[] _split = namespace.split("::");
        for(final String ns : _split) {
          _builder.append("namespace ");
          _builder.append(ns);
          _builder.append(" {");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append("namespace gen {");
      _builder.newLine();
      _builder.append("class ");
      String _effectiveName = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName);
      _builder.append("_regs;");
      _builder.newLineIfNotEmpty();
      _builder.append("}");
      _builder.newLine();
      _builder.newLine();
      _builder.append("class ");
      String _effectiveName_1 = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName_1);
      _builder.append(" : public sc_core::sc_module, public scc::tlm_target<> {");
      _builder.newLineIfNotEmpty();
      _builder.append("public:");
      _builder.newLine();
      _builder.append("    ");
      _builder.append("sc_core::sc_in<sc_core::sc_time> clk_i{\"clk_i\"};");
      _builder.newLine();
      _builder.append("    ");
      _builder.append("sc_core::sc_in<bool> rst_i{\"rst_i\"};");
      _builder.newLine();
      _builder.append("    ");
      String _effectiveName_2 = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName_2, "    ");
      _builder.append("(sc_core::sc_module_name nm);");
      _builder.newLineIfNotEmpty();
      _builder.append("    ");
      _builder.append("virtual ~");
      String _effectiveName_3 = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName_3, "    ");
      _builder.append("() override;");
      _builder.newLineIfNotEmpty();
      _builder.newLine();
      _builder.append("protected:");
      _builder.newLine();
      _builder.append("    ");
      _builder.append("void clock_cb();");
      _builder.newLine();
      _builder.append("    ");
      _builder.append("void reset_cb();");
      _builder.newLine();
      _builder.append("    ");
      _builder.append("sc_core::sc_time clk;");
      _builder.newLine();
      _builder.append("    ");
      _builder.append("std::unique_ptr<gen::");
      String _effectiveName_4 = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName_4, "    ");
      _builder.append("_regs> regs;");
      _builder.newLineIfNotEmpty();
      _builder.append("};");
      _builder.newLine();
      _builder.newLine();
      {
        List<String> _reverse = ListExtensions.<String>reverse(((List<String>)Conversions.doWrapArray(namespace.split("::"))));
        for(final String ns_1 : _reverse) {
          _builder.append("} // namespace ");
          _builder.append(ns_1);
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.newLine();
      _builder.append("#endif /* _");
      String _upperCase = namespace.toUpperCase();
      _builder.append(_upperCase);
      _builder.append("_");
      String _upperCase_1 = RdlUtil.effectiveName(this.componentDefinition).toUpperCase();
      _builder.append(_upperCase_1);
      _builder.append("_H_ */");
      _builder.newLineIfNotEmpty();
      _xblockexpression = _builder.toString();
    }
    return _xblockexpression;
  }
  
  @Override
  public String generateSource(final String namespace) {
    String _xblockexpression = null;
    {
      ComponentDefinitionType _type = this.componentDefinition.getType();
      boolean _notEquals = (!Objects.equal(_type, ComponentDefinitionType.REGFILE));
      if (_notEquals) {
        return "";
      }
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("/*");
      _builder.newLine();
      _builder.append(" ");
      _builder.append("* Copyright (c) 2019 -2022 MINRES Technologies GmbH");
      _builder.newLine();
      _builder.append(" ");
      _builder.append("*");
      _builder.newLine();
      _builder.append(" ");
      _builder.append("* SPDX-License-Identifier: Apache-2.0");
      _builder.newLine();
      _builder.append(" ");
      _builder.append("*/");
      _builder.newLine();
      _builder.newLine();
      _builder.append("#include \"");
      String _effectiveName = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName);
      _builder.append(".h\"");
      _builder.newLineIfNotEmpty();
      _builder.append("#include \"gen/");
      String _effectiveName_1 = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName_1);
      _builder.append(".h\"");
      _builder.newLineIfNotEmpty();
      _builder.newLine();
      _builder.append("#include <scc/utilities.h>");
      _builder.newLine();
      _builder.newLine();
      {
        String[] _split = namespace.split("::");
        for(final String ns : _split) {
          _builder.append("namespace ");
          _builder.append(ns);
          _builder.append(" {");
          _builder.newLineIfNotEmpty();
        }
      }
      _builder.append("SC_HAS_PROCESS(");
      String _effectiveName_2 = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName_2);
      _builder.append(");// NOLINT");
      _builder.newLineIfNotEmpty();
      _builder.newLine();
      String _effectiveName_3 = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName_3);
      _builder.append("::");
      String _effectiveName_4 = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName_4);
      _builder.append("(sc_core::sc_module_name nm)");
      _builder.newLineIfNotEmpty();
      _builder.append(": sc_core::sc_module(nm)");
      _builder.newLine();
      _builder.append(", scc::tlm_target<>(clk)");
      _builder.newLine();
      _builder.append(", NAMEDD(regs, gen::");
      String _effectiveName_5 = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName_5);
      _builder.append("_regs) {");
      _builder.newLineIfNotEmpty();
      _builder.append("    ");
      _builder.append("regs->registerResources(*this);");
      _builder.newLine();
      _builder.append("    ");
      _builder.append("SC_METHOD(clock_cb);");
      _builder.newLine();
      _builder.append("    ");
      _builder.append("sensitive << clk_i;");
      _builder.newLine();
      _builder.append("    ");
      _builder.append("SC_METHOD(reset_cb);");
      _builder.newLine();
      _builder.append("    ");
      _builder.append("sensitive << rst_i;");
      _builder.newLine();
      {
        EList<Instantiation> _instantiations = this.componentDefinition.getInstantiations();
        for(final Instantiation instantiation : _instantiations) {
          {
            EList<ComponentInstance> _componentInstances = instantiation.getComponentInstances();
            for(final ComponentInstance instance : _componentInstances) {
              {
                if ((Objects.equal(RdlUtil.getComponentDefinition(instantiation).getType(), ComponentDefinitionType.REG) && this.isReadOnly(RdlUtil.getComponentDefinition(instantiation)))) {
                  _builder.append("    ");
                  _builder.append("regs->");
                  String _name = instance.getName();
                  _builder.append(_name, "    ");
                  _builder.append(".set_write_cb([this](scc::sc_register<uint");
                  long _size = RdlUtil.getSize(instantiation);
                  _builder.append(_size, "    ");
                  _builder.append("_t>&, uint");
                  long _size_1 = RdlUtil.getSize(instantiation);
                  _builder.append(_size_1, "    ");
                  _builder.append("_t const& v, sc_core::sc_time t)-> bool {return true;});");
                  _builder.newLineIfNotEmpty();
                } else {
                  if ((Objects.equal(RdlUtil.getComponentDefinition(instantiation).getType(), ComponentDefinitionType.REG) && this.isWriteOnly(RdlUtil.getComponentDefinition(instantiation)))) {
                    _builder.append("    ");
                    _builder.append("regs->");
                    String _name_1 = instance.getName();
                    _builder.append(_name_1, "    ");
                    _builder.append(".set_read_cb([this](scc::sc_register<uint");
                    long _size_2 = RdlUtil.getSize(instantiation);
                    _builder.append(_size_2, "    ");
                    _builder.append("_t> const&, uint");
                    long _size_3 = RdlUtil.getSize(instantiation);
                    _builder.append(_size_3, "    ");
                    _builder.append("_t& v, sc_core::sc_time t)-> bool {return true;});");
                    _builder.newLineIfNotEmpty();
                  }
                }
              }
            }
          }
        }
      }
      _builder.append("}");
      _builder.newLine();
      _builder.newLine();
      String _effectiveName_6 = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName_6);
      _builder.append("::~");
      String _effectiveName_7 = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName_7);
      _builder.append("() {} // NOLINT");
      _builder.newLineIfNotEmpty();
      _builder.newLine();
      _builder.append("void ");
      String _effectiveName_8 = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName_8);
      _builder.append("::clock_cb() { this->clk = clk_i.read(); }");
      _builder.newLineIfNotEmpty();
      _builder.newLine();
      _builder.append("void ");
      String _effectiveName_9 = RdlUtil.effectiveName(this.componentDefinition);
      _builder.append(_effectiveName_9);
      _builder.append("::reset_cb() {");
      _builder.newLineIfNotEmpty();
      _builder.append("    ");
      _builder.append("if (rst_i.read()) {");
      _builder.newLine();
      _builder.append("        ");
      _builder.append("regs->reset_start();");
      _builder.newLine();
      _builder.append("    ");
      _builder.append("} else {");
      _builder.newLine();
      _builder.append("        ");
      _builder.append("regs->reset_stop();");
      _builder.newLine();
      _builder.append("    ");
      _builder.append("}");
      _builder.newLine();
      _builder.append("}");
      _builder.newLine();
      _builder.newLine();
      {
        List<String> _reverse = ListExtensions.<String>reverse(((List<String>)Conversions.doWrapArray(namespace.split("::"))));
        for(final String ns_1 : _reverse) {
          _builder.append("} // namespace ");
          _builder.append(ns_1);
          _builder.newLineIfNotEmpty();
        }
      }
      _xblockexpression = _builder.toString();
    }
    return _xblockexpression;
  }
  
  public boolean isReadOnly(final ComponentDefinition definition) {
    final Function1<PropertyAssignment, Boolean> _function = (PropertyAssignment it) -> {
      return Boolean.valueOf((it instanceof ExplicitPropertyAssignment));
    };
    final Function1<PropertyAssignment, ExplicitPropertyAssignment> _function_1 = (PropertyAssignment it) -> {
      return ((ExplicitPropertyAssignment) it);
    };
    final Function1<ExplicitPropertyAssignment, Boolean> _function_2 = (ExplicitPropertyAssignment it) -> {
      return Boolean.valueOf((((Objects.equal(it.getName(), PropertyEnum.SW) && (it.getRhs().getValue() != null)) && (it.getRhs().getValue().getVal() != null)) && (it.getRhs().getValue().getVal() == RValueConstant.R)));
    };
    int _size = IterableExtensions.size(IterableExtensions.<ExplicitPropertyAssignment>filter(IterableExtensions.<PropertyAssignment, ExplicitPropertyAssignment>map(IterableExtensions.<PropertyAssignment>filter(definition.getPropertyAssignments(), _function), _function_1), _function_2));
    return (_size > 0);
  }
  
  public boolean isWriteOnly(final ComponentDefinition definition) {
    final Function1<PropertyAssignment, Boolean> _function = (PropertyAssignment it) -> {
      return Boolean.valueOf((it instanceof ExplicitPropertyAssignment));
    };
    final Function1<PropertyAssignment, ExplicitPropertyAssignment> _function_1 = (PropertyAssignment it) -> {
      return ((ExplicitPropertyAssignment) it);
    };
    final Function1<ExplicitPropertyAssignment, Boolean> _function_2 = (ExplicitPropertyAssignment it) -> {
      return Boolean.valueOf((((Objects.equal(it.getName(), PropertyEnum.SW) && (it.getRhs().getValue() != null)) && (it.getRhs().getValue().getVal() != null)) && (it.getRhs().getValue().getVal() == RValueConstant.W)));
    };
    int _size = IterableExtensions.size(IterableExtensions.<ExplicitPropertyAssignment>filter(IterableExtensions.<PropertyAssignment, ExplicitPropertyAssignment>map(IterableExtensions.<PropertyAssignment>filter(definition.getPropertyAssignments(), _function), _function_1), _function_2));
    return (_size > 0);
  }
}
